#ifndef CONTROL_INCLUDED
#define CONTROL_INCLUDED

#include "option.h"
#include "grammar.h"
#include "node.h"
#include "base.h"
#include "pda.h"

//
//
//
class Control
{
public:

    Control(Option *option_,
            LexStream *lex_stream_,
            VariableLookupTable *variable_table_,
            MacroLookupTable *macro_table_) : option(option_)
    {
        syslis = fopen(option -> lis_file, "w");
        if (syslis  == (FILE *) NULL)
        {
            fprintf(stderr, "***ERROR: Listing file \"%s\" cannot be openned.\n",
                    option -> lis_file);
            exit(12);
        }

        //
        // Flush buffer to print options and initial reports
        //
        if (! option -> quiet)
        {
            option -> report.Print(syslis);
            option -> report.Flush(stdout);
        }

        node_pool = new NodePool();
        grammar = new Grammar(this, &(option -> ActionBlocks()), lex_stream_, variable_table_, macro_table_);
        base = new Base(this);
        pda = new Pda(this, base);
    }

    //
    // Close listing file and destroy the objects allocated in the constructor.
    //
    ~Control()
    {
        fclose(syslis); // close listing file

        delete node_pool;
        delete grammar;
        delete base;
        delete pda;
    }

    enum
    {
        SYMBOL_SIZE = 256,
        PRINT_LINE_SIZE = 80
    };

    static const char HEADER_INFO[],
                      VERSION[];

    void PrintHeading(int code = 0)
    {
        fprintf(syslis, "\f\n\n %-39s%s\n\n", HEADER_INFO, VERSION);
        return;
    }

    Option *option;

    NodePool *node_pool;
    Grammar *grammar;
    Base *base;
    Pda *pda;
    static FILE *syslis;

    void ConstructParser(void);
};
#endif /* CONTROL_INCLUDED */
