#include "symbol.h"
#include "option.h"
#include "scanner.h"
#include "option.h"
#include "control.h"

#include <assert.h>
#include <new>
using namespace std;

//
// If the system runs out of memory, this function is invoked
//
void OutOfMemory()
{
    cerr << "***System Failure: Out of memory\n";
    exit(1);
}


//
//
//
int main(int argc, char *argv[])
{
    //
    // If only "lpg" or "lpg ?*" is typed, we display the help
    // screen.
    //
    if (argc == 1 || argv[1][0] == '?' || strcmp(argv[1],"-h") == 0)
    {
        Option::PrintOptionsList();
        return 4;
    }
    else
    {
	    cout << "\n"
	         << Control::HEADER_INFO
    		 << " Version " << Control::VERSION
	         << "\n(C) Copyright IBM Corp. 1984, 2006.\n\n";
    }

    set_new_handler(OutOfMemory);

    Option option(argc, (const char **) argv);
    LexStream lex_stream(&option);
    VariableLookupTable variable_table;
    MacroLookupTable *macro_table = new MacroLookupTable();
    Scanner *scanner = new Scanner(&option, &lex_stream, &variable_table, macro_table);
    scanner -> Scan();
    delete scanner;
#ifdef TEST
    lex_stream.Dump(); // TODO: REMOVE THIS !!!
#endif
    if (lex_stream.NumTokens() == 0 || lex_stream.NumBadTokens() > 0)
    {
        delete macro_table;
    }
    else
    {
        Control control(&option, &lex_stream, &variable_table, macro_table);
        control.grammar -> Process();

        delete macro_table;

        control.ConstructParser();
    }

    return 0;
}
